@@ -225,13 +225,18 @@ typedef enum {
 
 	GTK_HTML_COMMAND_BLOCK_SELECTION,
 	GTK_HTML_COMMAND_UNBLOCK_SELECTION,
+
+	GTK_HTML_COMMAND_IS_SELECTION_ACTIVE,
+	GTK_HTML_COMMAND_SCROLL_FORWARD,
+	GTK_HTML_COMMAND_SCROLL_BACKWARD,
 } GtkHTMLCommandType;
 
 typedef enum {
 	GTK_HTML_CURSOR_SKIP_ONE,
 	GTK_HTML_CURSOR_SKIP_WORD,
 	GTK_HTML_CURSOR_SKIP_PAGE,
-	GTK_HTML_CURSOR_SKIP_ALL
+	GTK_HTML_CURSOR_SKIP_ALL,
+	GTK_HTML_CURSOR_SKIP_NONE
 } GtkHTMLCursorSkipType;
 
 typedef enum {
