@@ -2276,17 +2276,19 @@ search_text (HTMLObject **beg, HTMLSearch *info)
 		/* set eq_bytes and pos counters */
 		eq_bytes = 0;
 		if (info->found) {
-                	if (info->start_pos > 0)
-	                        index = ((guchar *)g_utf8_offset_to_pointer ((gchar *) par, info->start_pos + ((info->forward) ? 1 : -1))) - par;
+			if (info->start_pos > 0)
+				index = ((guchar *)g_utf8_offset_to_pointer ((gchar *) par, info->start_pos + ((info->forward) ? 0 : -1))) - par;
 			else
-				index = ((guchar *)g_utf8_offset_to_pointer ((gchar *) par, info->start_pos + ((info->forward) ? 1 : 0))) - par;
+				index = ((guchar *)g_utf8_offset_to_pointer ((gchar *) par, info->start_pos)) - par;
 		} else {
 			index = (info->forward) ? 0 : text_bytes - 1;
 		}
 
 		/* FIXME make shorter text instead */
-		if (!info->forward)
-			par [index+1] = 0;
+		if (!info->forward) {
+			guchar* tmp = (guchar *)g_utf8_next_char (par + index);
+			*tmp = '\0';
+		}
 
 		if ((info->forward && index < text_bytes)
 		    || (!info->forward && index > 0)) {
@@ -2318,7 +2320,7 @@ search_text (HTMLObject **beg, HTMLSearch *info)
 						break;
 					}
 					index += (info->forward)
-						? (((guchar *) g_utf8_next_char (par + index)) - par - index)
+						? (((guchar *) g_utf8_next_char ((gchar *) par + index)) - par - index)
 						: (((guchar *) g_utf8_prev_char ((gchar *) par + index)) - par - index);
 				}
 #else
@@ -2342,16 +2344,25 @@ search_text (HTMLObject **beg, HTMLSearch *info)
 				/* substring search - simple one - could be improved
 				   go thru par and look for info->text */
 				while (par [index]) {
-					if (info->trans [(guchar) info->text
-							[(info->forward) ? eq_bytes : info->text_bytes - eq_bytes - 1]]
-					    == info->trans [par [index]]) {
-						eq_bytes ++;
+					gunichar unicode_info, unicode_par;
+
+					unicode_info = g_utf8_get_char (((info->forward)
+									? (info->text + eq_bytes)
+									: g_utf8_prev_char (info->text + info->text_bytes - eq_bytes)));
+					unicode_par = g_utf8_get_char ((gchar *) par + index);
+					if (!info->case_sensitive) {
+                                             unicode_info = g_unichar_toupper (unicode_info);
+				 	     unicode_par  = g_unichar_toupper (unicode_par);
+					}
+
+               				if (unicode_info == unicode_par) {
+						eq_bytes += (guchar *)g_utf8_next_char (par + index) - par - index;
+
 						if (eq_bytes == info->text_bytes) {
-							search_set_info (head, info, par,
-									 index - (info->forward
-										  ? -(((guchar *) g_utf8_next_char (par + index - eq_bytes)) - par - index)
-										  : 0),
-									 info->text_bytes);
+							/* The above par + index is always at the beginning of the last character matched */
+							if (info->forward)
+								index = (guchar *) g_utf8_next_char ((gchar *) par + index) - par - eq_bytes;
+							search_set_info (head, info, par, index, info->text_bytes);
 							retval=TRUE;
 							break;
 						}
