@@ -23,11 +23,9 @@
 #include "gtkhtml-compat.h"
 
 #include <string.h>
+#include <gtk/gtk.h>
 #include <gnome.h>
 #include <ctype.h>
-#include <libgnomeprint/gnome-print.h>
-#include <libgnomeprint/gnome-print-pango.h>
-#include <libgnomeprint/gnome-print-paper.h>
 
 #include "htmlembedded.h"
 #include "gtkhtml-embedded.h"
@@ -38,95 +36,28 @@
 /* #define PRINTER_DEBUG */
 
 
-static HTMLPainterClass *parent_class = NULL;
+static gpointer parent_class = NULL;
 
 
 /* The size of a pixel in the printed output, in points.  */
 #define PIXEL_SIZE .5
 
-static void
-insure_config (HTMLPrinter *p)
-{
-	if (!p->config)
-		p->config = p->master ? gnome_print_job_get_config (p->master) : gnome_print_config_default ();
-}
-
 static gdouble
 printer_get_page_height (HTMLPrinter *printer)
 {
-	gdouble width, height = 0.0;
+	GtkPageSetup *page_setup;
 
-	insure_config (printer);
-	gnome_print_config_get_page_size (printer->config, &width, &height);
-
-	return height;
+	page_setup = gtk_print_context_get_page_setup (printer->context);
+	return gtk_page_setup_get_page_height (page_setup, GTK_UNIT_POINTS);
 }
 
 static gdouble
 printer_get_page_width (HTMLPrinter *printer)
 {
-	gdouble width = 0.0, height;
+	GtkPageSetup *page_setup;
 
-	insure_config (printer);
-	gnome_print_config_get_page_size (printer->config, &width, &height);
-
-	return width;
-}
-
-/* .5" in PS units */
-#define TEMP_MARGIN 72*.5
-
-static gdouble
-get_lmargin (HTMLPrinter *printer)
-{
-	/*gdouble lmargin = 0.0;
-
-	insure_config (printer);
-	gnome_print_config_get_double (printer->config, GNOME_PRINT_KEY_PAGE_MARGIN_LEFT, &lmargin);
-
-	printf ("lmargin %f\n", lmargin);
-	return lmargin;*/
-
-	return TEMP_MARGIN;
-}
-
-static gdouble
-get_rmargin (HTMLPrinter *printer)
-{
-	/*gdouble rmargin = 0.0;
-
-	insure_config (printer);
-	gnome_print_config_get_double (printer->config, GNOME_PRINT_KEY_PAGE_MARGIN_RIGHT, &rmargin);
-
-	return rmargin;*/
-
-	return TEMP_MARGIN;
-}
-
-static gdouble
-get_tmargin (HTMLPrinter *printer)
-{
-	/* gdouble tmargin = 0.0;
-
-	insure_config (printer);
-	gnome_print_config_get_double (printer->config, GNOME_PRINT_KEY_PAGE_MARGIN_TOP, &tmargin);
-
-	return tmargin; */
-
-	return TEMP_MARGIN;
-}
-
-static gdouble
-get_bmargin (HTMLPrinter *printer)
-{
-	/* gdouble bmargin = 0.0;
-
-	insure_config (printer);
-	gnome_print_config_get_double (printer->config, GNOME_PRINT_KEY_PAGE_MARGIN_BOTTOM, &bmargin);
-
-	return bmargin; */
-
-	return TEMP_MARGIN;
+	page_setup = gtk_print_context_get_page_setup (printer->context);
+	return gtk_page_setup_get_page_width (page_setup, GTK_UNIT_POINTS);
 }
 
 gdouble
@@ -135,37 +66,6 @@ html_printer_scale_to_gnome_print (HTMLPrinter *printer, gint x)
 	return SCALE_ENGINE_TO_GNOME_PRINT (x);
 }
 
-void
-html_printer_coordinates_to_gnome_print (HTMLPrinter *printer,
-					 gint engine_x, gint engine_y,
-					 gdouble *print_x_return, gdouble *print_y_return)
-{
-	gdouble print_x, print_y;
-
-	print_x = SCALE_ENGINE_TO_GNOME_PRINT (engine_x);
-	print_y = SCALE_ENGINE_TO_GNOME_PRINT (engine_y);
-
-	print_x = print_x + get_lmargin (printer);
-	print_y = (printer_get_page_height (printer) - print_y) - get_tmargin (printer);
-
-	*print_x_return = print_x;
-	*print_y_return = print_y;
-}
-
-#if 0
-static void
-gnome_print_coordinates_to_engine (HTMLPrinter *printer,
-				   gdouble print_x, gdouble print_y,
-				   gint *engine_x_return, gint *engine_y_return)
-{
-	print_x -= get_lmargin (printer);
-	print_y -= get_bmargin (printer);
-
-	*engine_x_return = SCALE_ENGINE_TO_GNOME_PRINT (print_x);
-	*engine_y_return = SCALE_GNOME_PRINT_TO_ENGINE (get_page_height (printer) - print_y);
-}
-#endif
-
 
 /* GtkObject methods.  */
 
@@ -176,17 +76,12 @@ finalize (GObject *object)
 
 	printer = HTML_PRINTER (object);
 
-	if (printer->config != NULL) {
-		/* FIXME g_object_unref (printer->config); */
-		printer->config = NULL;
-	}
 	if (printer->context != NULL) {
-		gnome_print_context_close (printer->context);
-		g_object_unref (printer->context);
+		/* FIXME g_object_unref (printer->context);*/
 		printer->context = NULL;
 	}
 
-	(* G_OBJECT_CLASS (parent_class)->finalize) (object);
+	G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
 
@@ -196,9 +91,10 @@ begin (HTMLPainter *painter,
        int x2, int y2)
 {
 	HTMLPrinter *printer;
-	GnomePrintContext *pc;
+	GtkPrintContext *pc;
 	gdouble printer_x1, printer_y1;
 	gdouble printer_x2, printer_y2;
+	cairo_t *cr;
 #ifdef PRINTER_DEBUG
 	gdouble dash [2];
 #endif
@@ -207,45 +103,33 @@ begin (HTMLPainter *painter,
 	pc      = printer->context;
 	g_return_if_fail (pc);
 
-	gnome_print_beginpage (pc, "page");
-	gnome_print_gsave (pc);
+	cr = gtk_print_context_get_cairo_context (pc);
+	cairo_save (cr);
+ 
+	printer_x1 = SCALE_ENGINE_TO_GNOME_PRINT (x1);
+	printer_y1 = SCALE_ENGINE_TO_GNOME_PRINT (y1);
 
-	html_printer_coordinates_to_gnome_print (printer, x1, y1, &printer_x1, &printer_y1);
 	printer_x2 = printer_x1 + SCALE_ENGINE_TO_GNOME_PRINT (x2);
-	printer_y2 = printer_y1 - SCALE_ENGINE_TO_GNOME_PRINT (y2);
-
-	gnome_print_newpath (pc);
-
-	gnome_print_moveto (pc, printer_x1, printer_y1);
-	gnome_print_lineto (pc, printer_x1, printer_y2);
-	gnome_print_lineto (pc, printer_x2, printer_y2);
-	gnome_print_lineto (pc, printer_x2, printer_y1);
-	gnome_print_lineto (pc, printer_x1, printer_y1);
-	gnome_print_closepath (pc);
+	printer_y2 = SCALE_ENGINE_TO_GNOME_PRINT (y2);
 
+	cairo_rectangle (cr, printer_x1, printer_y1, printer_x2, printer_y2); 
 #ifdef PRINTER_DEBUG
-	gnome_print_gsave (pc);
+	cairo_save (cr);
 	dash [0] = 10.0;
 	dash [1] = 10.0;
-	gnome_print_setrgbcolor (pc, .5, .5, .5);
-	gnome_print_setlinewidth (pc, .3);
-	gnome_print_setdash (pc, 2, dash, .0);
-	gnome_print_stroke (pc);
-	gnome_print_grestore (pc);
+	cairo_set_source_rgb (cr, .5, .5, .5);
+	cairo_set_line_width (cr, .3);
+	cairo_set_dash (cr, 2, dash, .0);
+	cairo_stroke (cr);
+	cairo_restore (cr);
 #endif
-	gnome_print_clip (pc);
+	cairo_clip (cr);
+	cairo_restore (cr);
 }
 
 static void
 end (HTMLPainter *painter)
 {
-	HTMLPrinter *printer;
-
-	printer = HTML_PRINTER (painter);
-	g_return_if_fail (printer->context != NULL);
-
-	gnome_print_grestore (printer->context);
-	gnome_print_showpage (printer->context);
 }
 
 static void
@@ -270,12 +154,13 @@ set_pen (HTMLPainter *painter,
 	 const GdkColor *color)
 {
 	HTMLPrinter *printer;
+	cairo_t *cr;
 
 	printer = HTML_PRINTER (painter);
 	g_return_if_fail (printer->context != NULL);
 
-	gnome_print_setrgbcolor (printer->context,
-				 color->red / 65535.0, color->green / 65535.0, color->blue / 65535.0);
+	cr = gtk_print_context_get_cairo_context (printer->context);
+	cairo_set_source_rgb (cr, color->red / 65535.0, color->green / 65535.0, color->blue / 65535.0);
 }
 
 static const GdkColor *
@@ -290,35 +175,34 @@ static void
 prepare_rectangle (HTMLPainter *painter, gint _x, gint _y, gint w, gint h)
 {
 	HTMLPrinter *printer = HTML_PRINTER (painter);
-	GnomePrintContext *context = printer->context;
+	GtkPrintContext *context = printer->context;
 	gdouble x;
 	gdouble y;
 	gdouble width;
 	gdouble height;
+	cairo_t *cr;
 
 	width = SCALE_ENGINE_TO_GNOME_PRINT (w);
 	height = SCALE_ENGINE_TO_GNOME_PRINT (h);
-
-	html_printer_coordinates_to_gnome_print (HTML_PRINTER (painter), _x, _y, &x, &y);
-
-	gnome_print_newpath (context);
-	gnome_print_moveto  (context, x, y);
-	gnome_print_lineto  (context, x + width, y);
-	gnome_print_lineto  (context, x + width, y - height);
-	gnome_print_lineto  (context, x, y - height);
-	gnome_print_lineto  (context, x, y);
-	gnome_print_closepath (context);
+	x = SCALE_ENGINE_TO_GNOME_PRINT (_x);
+	y = SCALE_ENGINE_TO_GNOME_PRINT (_y);
+	cr = gtk_print_context_get_cairo_context (context);
+	cairo_new_path (cr);
+	cairo_rectangle (cr, x, y, x + width, y + height );
+	cairo_close_path (cr);
 }
 
 static void
 do_rectangle (HTMLPainter *painter, gint x, gint y, gint w, gint h, gint lw)
 {
 	HTMLPrinter *printer = HTML_PRINTER (painter);
-	GnomePrintContext *context = printer->context;
+	GtkPrintContext *context = printer->context;
+	cairo_t *cr;
 
-	gnome_print_setlinewidth (context, SCALE_ENGINE_TO_GNOME_PRINT (lw) * PIXEL_SIZE);
-	prepare_rectangle (painter, x, y, w, h);
-	gnome_print_stroke (context);
+	cr = gtk_print_context_get_cairo_context (context);
+	cairo_set_line_width (cr, SCALE_ENGINE_TO_GNOME_PRINT (lw) * PIXEL_SIZE); 
+	prepare_rectangle (painter,x, y, w, h);
+	cairo_stroke (cr);
 }
 
 static void
@@ -326,8 +210,10 @@ set_clip_rectangle (HTMLPainter *painter,
 		    gint x, gint y,
 		    gint width, gint height)
 {
+	cairo_t *cr;
 	prepare_rectangle (painter, x, y, width, height);
-	gnome_print_clip (HTML_PRINTER (painter)->context);
+	cr = gtk_print_context_get_cairo_context (HTML_PRINTER (painter)->context);
+	cairo_clip (cr);
 }
 
 /* HTMLPainter drawing functions.  */
@@ -340,20 +226,23 @@ draw_line (HTMLPainter *painter,
 	HTMLPrinter *printer;
 	double printer_x1, printer_y1;
 	double printer_x2, printer_y2;
+	cairo_t *cr;
 
 	printer = HTML_PRINTER (painter);
 	g_return_if_fail (printer->context != NULL);
 
-	html_printer_coordinates_to_gnome_print (printer, x1, y1, &printer_x1, &printer_y1);
-	html_printer_coordinates_to_gnome_print (printer, x2, y2, &printer_x2, &printer_y2);
+	printer_x1 = SCALE_ENGINE_TO_GNOME_PRINT (x1);
+	printer_y1 = SCALE_ENGINE_TO_GNOME_PRINT (y1);
+	printer_x2 = SCALE_ENGINE_TO_GNOME_PRINT (x2);
+	printer_y2 = SCALE_ENGINE_TO_GNOME_PRINT (y2);
 
-	gnome_print_setlinewidth (printer->context, PIXEL_SIZE);
+	cr = gtk_print_context_get_cairo_context (printer->context);
+	cairo_set_line_width (cr, PIXEL_SIZE); 
 
-	gnome_print_newpath (printer->context);
-	gnome_print_moveto (printer->context, printer_x1, printer_y1);
-	gnome_print_lineto (printer->context, printer_x2, printer_y2);
-
-	gnome_print_stroke (printer->context);
+	cairo_new_path (cr);
+	cairo_move_to (cr, printer_x1, printer_y1);
+	cairo_line_to (cr, printer_x2, printer_y2);
+	cairo_stroke (cr);
 }
 
 static void
@@ -373,13 +262,14 @@ draw_border (HTMLPainter *painter,
 	     gint bordersize)
 {
 	HTMLPrinter *printer = HTML_PRINTER (painter);
-	GnomePrintContext *pc = printer->context;
+	GtkPrintContext *pc = printer->context;
 	GdkColor *col1 = NULL, *col2 = NULL;
 	GdkColor dark, light;
 	gdouble x;
 	gdouble y;
 	gdouble width, bs;
 	gdouble height;
+	cairo_t *cr;
 
 	#define INC 0x8000
 	#define DARK(c)  dark.c = MAX (((gint) bg->c) - INC, 0)
@@ -414,33 +304,34 @@ draw_border (HTMLPainter *painter,
 	height = SCALE_ENGINE_TO_GNOME_PRINT (h);
 	bs     = SCALE_ENGINE_TO_GNOME_PRINT (bordersize);
 
-	html_printer_coordinates_to_gnome_print (HTML_PRINTER (painter), _x, _y, &x, &y);
+	x = SCALE_ENGINE_TO_GNOME_PRINT (_x);
+	y = SCALE_ENGINE_TO_GNOME_PRINT (_y);
 
+	cr = gtk_print_context_get_cairo_context (pc);
 	if (col2)
-		gnome_print_setrgbcolor (pc, col1->red / 65535.0, col1->green / 65535.0, col1->blue / 65535.0);
-
-	gnome_print_newpath (pc);
-	gnome_print_moveto  (pc, x, y);
-	gnome_print_lineto  (pc, x + width, y);
-	gnome_print_lineto  (pc, x + width - bs, y - bs);
-	gnome_print_lineto  (pc, x + bs, y - bs );
-	gnome_print_lineto  (pc, x + bs, y - height + bs);
-	gnome_print_lineto  (pc, x, y - height);
-	gnome_print_closepath (pc);
-	gnome_print_fill    (pc);
+		cairo_set_source_rgb (cr, col1->red / 65535.0, col1->green / 65535.0, col1->blue / 65535.0);
+	cairo_new_path (cr);
+	cairo_move_to  (cr, x, y);
+	cairo_line_to  (cr, x + width, y);
+	cairo_line_to  (cr, x + width - bs, y + bs);
+	cairo_line_to  (cr, x + bs, y + bs );
+	cairo_line_to  (cr, x + bs, y + height - bs);
+	cairo_line_to  (cr, x, y + height);
+	cairo_close_path (cr);
+	cairo_fill (cr);
 
 	if (col1)
-		gnome_print_setrgbcolor (pc, col2->red / 65535.0, col2->green / 65535.0, col2->blue / 65535.0);
+		cairo_set_source_rgb (cr, col2->red / 65535.0, col2->green / 65535.0, col2->blue / 65535.0);
 
-	gnome_print_newpath (pc);
-	gnome_print_moveto  (pc, x, y - height);
-	gnome_print_lineto  (pc, x + width, y - height);
-	gnome_print_lineto  (pc, x + width, y);
-	gnome_print_lineto  (pc, x + width - bs, y - bs);
-	gnome_print_lineto  (pc, x + width - bs, y - height + bs);
-	gnome_print_lineto  (pc, x + bs, y - height + bs);
-	gnome_print_closepath (pc);
-	gnome_print_fill    (pc);
+	cairo_new_path (cr);
+	cairo_move_to  (cr, x, y + height);
+	cairo_line_to  (cr, x + width, y + height);
+	cairo_line_to  (cr, x + width, y);
+	cairo_line_to  (cr, x + width - bs, y + bs);
+	cairo_line_to  (cr, x + width - bs, y + height - bs);
+	cairo_line_to  (cr, x + bs, y + height - bs); 
+	cairo_close_path (cr);
+	cairo_fill (cr);
 }
 
 static void
@@ -451,10 +342,11 @@ draw_background (HTMLPainter *painter,
 		 gint pix_width, gint pix_height,
 		 gint tile_x, gint tile_y)
 {
-	GnomePrintContext *pc;
+	GtkPrintContext *pc;
 	HTMLPrinter *printer;
 	gdouble x, y, width, height;
-
+	cairo_t *cr;
+  
 	printer = HTML_PRINTER (painter);
 	g_return_if_fail (printer);
 	pc = printer->context;
@@ -462,41 +354,39 @@ draw_background (HTMLPainter *painter,
 
 	width = SCALE_ENGINE_TO_GNOME_PRINT  (pix_width);
 	height = SCALE_ENGINE_TO_GNOME_PRINT (pix_height);
-	html_printer_coordinates_to_gnome_print (printer, ix, iy, &x, &y);
+	x = SCALE_ENGINE_TO_GNOME_PRINT (ix);
+	y = SCALE_ENGINE_TO_GNOME_PRINT (iy);
 
 	if (color) {
-		gnome_print_setrgbcolor (pc, color->red / 65535.0, color->green / 65535.0, color->blue / 65535.0);
-
-		gnome_print_newpath (pc);
-		gnome_print_moveto (pc, x, y);
-		gnome_print_lineto (pc, x + width, y);
-		gnome_print_lineto (pc, x + width, y - height);
-		gnome_print_lineto (pc, x, y - height);
-		gnome_print_lineto (pc, x, y);
-		gnome_print_closepath (pc);
-
-		gnome_print_fill (pc);
+		cr = gtk_print_context_get_cairo_context (pc);
+		cairo_save (cr);
+		cairo_set_source_rgb (cr, color->red / 65535.0, color->green / 65535.0, color->blue / 65535.0);
+		cairo_new_path (cr);
+		cairo_rectangle (cr, x, y, width, height);
+		cairo_close_path (cr);
+		cairo_fill (cr); 
+		cairo_restore (cr);
 	}
 }
 
 static void
-print_pixbuf (GnomePrintContext *pc, GdkPixbuf *pixbuf)
+print_pixbuf (GtkPrintContext *pc, GdkPixbuf *pixbuf)
 {
+	cairo_t *cr;
 	if (!pixbuf || (gdk_pixbuf_get_colorspace (pixbuf) != GDK_COLORSPACE_RGB))
 		return;
 	
+	cr = gtk_print_context_get_cairo_context (pc);
 	if (gdk_pixbuf_get_has_alpha (pixbuf)) {
-		gnome_print_rgbaimage (pc, 
-				       gdk_pixbuf_get_pixels (pixbuf),
-				       gdk_pixbuf_get_width (pixbuf),
-				       gdk_pixbuf_get_height (pixbuf),
-				       gdk_pixbuf_get_rowstride (pixbuf));
+		gdk_cairo_set_source_pixbuf (cr, pixbuf, 0, 0); 
+		cairo_rectangle (cr, 0, 0, (double)gdk_pixbuf_get_width (pixbuf), (double)gdk_pixbuf_get_height (pixbuf));                 cairo_clip (cr);
+		cairo_paint (cr);
+		
 	} else {
-		gnome_print_rgbimage (pc, 
-				      gdk_pixbuf_get_pixels (pixbuf),
-				      gdk_pixbuf_get_width (pixbuf),
-				      gdk_pixbuf_get_height (pixbuf),
-				      gdk_pixbuf_get_rowstride (pixbuf));
+		gdk_cairo_set_source_pixbuf (cr, pixbuf, 0, 0);
+		cairo_rectangle (cr, 0, 0, (double)gdk_pixbuf_get_width (pixbuf), (double)gdk_pixbuf_get_height (pixbuf));
+		cairo_clip (cr);
+		cairo_paint (cr);
 	}
 }
 
@@ -506,20 +396,22 @@ draw_pixmap (HTMLPainter *painter, GdkPixbuf *pixbuf, gint x, gint y, gint scale
 	HTMLPrinter *printer;
 	double print_x, print_y;
 	double print_scale_width, print_scale_height;
+	double page_height;
+	cairo_t *cr;
 
 	printer = HTML_PRINTER (painter);
-	g_return_if_fail (printer->context != NULL);
-
-	html_printer_coordinates_to_gnome_print (printer, x, y, &print_x, &print_y);
-
+       	g_return_if_fail (printer->context != NULL);
+	cr = gtk_print_context_get_cairo_context (printer->context);
+	page_height =(double) gtk_print_context_get_height (printer->context);
+	print_x = SCALE_ENGINE_TO_GNOME_PRINT (x);
+	print_y = SCALE_ENGINE_TO_GNOME_PRINT (y);
 	print_scale_width  = SCALE_ENGINE_TO_GNOME_PRINT (scale_width);
 	print_scale_height = SCALE_ENGINE_TO_GNOME_PRINT (scale_height);
-
-	gnome_print_gsave (printer->context);
-	gnome_print_translate (printer->context, print_x, print_y - print_scale_height);
-	gnome_print_scale (printer->context, print_scale_width, print_scale_height);
+	cairo_save (cr);
+	cairo_translate (cr, print_x, print_y);
+	cairo_scale (cr, print_scale_width / (double)gdk_pixbuf_get_width (pixbuf), print_scale_height / (double)gdk_pixbuf_get_height (pixbuf));
 	print_pixbuf (printer->context, pixbuf);
-	gnome_print_grestore (printer->context);
+	cairo_restore (cr);  
 }
 
 static void
@@ -528,6 +420,7 @@ fill_rect (HTMLPainter *painter, gint x, gint y, gint width, gint height)
 	HTMLPrinter *printer;
 	double printer_x, printer_y;
 	double printer_width, printer_height;
+	cairo_t *cr;
 
 	printer = HTML_PRINTER (painter);
 	g_return_if_fail (printer->context != NULL);
@@ -535,64 +428,61 @@ fill_rect (HTMLPainter *painter, gint x, gint y, gint width, gint height)
 	printer_width = SCALE_ENGINE_TO_GNOME_PRINT (width);
 	printer_height = SCALE_ENGINE_TO_GNOME_PRINT (height);
 
-	html_printer_coordinates_to_gnome_print (printer, x, y, &printer_x, &printer_y);
+	printer_x = SCALE_ENGINE_TO_GNOME_PRINT (x);
+	printer_y = SCALE_ENGINE_TO_GNOME_PRINT (y);
 
-	gnome_print_newpath (printer->context);
-	gnome_print_moveto (printer->context, printer_x, printer_y);
-	gnome_print_lineto (printer->context, printer_x + printer_width, printer_y);
-	gnome_print_lineto (printer->context, printer_x + printer_width, printer_y - printer_height);
-	gnome_print_lineto (printer->context, printer_x, printer_y - printer_height);
-	gnome_print_lineto (printer->context, printer_x, printer_y);
-	gnome_print_closepath (printer->context);
-
-	gnome_print_fill (printer->context);
+	cr = gtk_print_context_get_cairo_context (printer->context);
+	cairo_new_path (cr);
+	cairo_rectangle (cr, printer_x, printer_y, printer_width, printer_height);
+	cairo_close_path (cr);
+	cairo_fill (cr);
 }
 
 static void
 draw_lines (HTMLPrinter *printer, double x, double y, double width, PangoAnalysis *analysis, HTMLPangoProperties *properties)
 {
 	PangoFontMetrics *metrics;
+	cairo_t *cr;
 	
 	if (!properties->underline && !properties->strikethrough)
 		return;
 	
 	metrics = pango_font_get_metrics (analysis->font, analysis->language);
-
-	gnome_print_setlinecap (printer->context, GDK_CAP_BUTT);
+	cr = gtk_print_context_get_cairo_context (printer->context);
+	cairo_set_line_cap (cr, CAIRO_LINE_CAP_BUTT);
 
 	if (properties->underline) {
 #ifdef PANGO_1_5_OR_HIGHER
-		double thickness = SCALE_PANGO_TO_GNOME_PRINT (pango_font_metrics_get_underline_thickness (metrics));
-		double position = SCALE_PANGO_TO_GNOME_PRINT (pango_font_metrics_get_underline_position (metrics));
+		double thickness = pango_units_to_double (pango_font_metrics_get_underline_thickness (metrics));
+		double position = pango_units_to_double (pango_font_metrics_get_underline_position (metrics));
 		double ly = y + position - thickness / 2;
 #else
 		double thickness = 1.0;
 		double position = -1.0;
 		double ly = y + position - thickness / 2;
 #endif
-
-		gnome_print_newpath (printer->context);
-		gnome_print_moveto (printer->context, x, ly);
-		gnome_print_lineto (printer->context, x + width, ly);
-		gnome_print_setlinewidth (printer->context, thickness);
-		gnome_print_stroke (printer->context);
+	    	cairo_new_path (cr);
+		cairo_move_to (cr, x, ly + 4);
+		cairo_line_to (cr, x + width, ly + 4);
+		cairo_set_line_width (cr, thickness);
+		cairo_stroke (cr);
 	}
 		
 	if (properties->strikethrough) {
 #ifdef PANGO_1_5_OR_HIGHER
-		double thickness = SCALE_PANGO_TO_GNOME_PRINT (pango_font_metrics_get_strikethrough_thickness (metrics));
-		double position = SCALE_PANGO_TO_GNOME_PRINT (pango_font_metrics_get_strikethrough_position (metrics));
+		double thickness = pango_units_to_double (pango_font_metrics_get_strikethrough_thickness (metrics));
+		double position = pango_units_to_double (pango_font_metrics_get_strikethrough_position (metrics));
 		double ly = y + position - thickness / 2;
 #else
 		double thickness = 1.0;
-		double position = SCALE_PANGO_TO_GNOME_PRINT (pango_font_metrics_get_ascent (metrics)/3);
+		double position = pango_units_to_double (pango_font_metrics_get_ascent (metrics)/3);
 		double ly = y + position - thickness / 2;
 #endif
-		gnome_print_newpath (printer->context);
-		gnome_print_moveto (printer->context, x, ly);
-		gnome_print_lineto (printer->context, x + width, ly);
-		gnome_print_setlinewidth (printer->context, thickness);
-		gnome_print_stroke (printer->context);
+		cairo_new_path (cr);
+		cairo_move_to (cr, x, ly - 8);
+		cairo_line_to (cr, x + width, ly - 8);
+		cairo_set_line_width (cr, thickness);
+		cairo_stroke (cr);
 	}
 }
 
@@ -603,49 +493,50 @@ draw_glyphs (HTMLPainter *painter, gint x, gint y, PangoItem *item, PangoGlyphSt
 	gdouble print_x, print_y;
 	PangoRectangle log_rect;
 	HTMLPangoProperties properties;
+	cairo_t *cr;
 
 	printer = HTML_PRINTER (painter);
 
-	html_printer_coordinates_to_gnome_print (printer, x, y, &print_x, &print_y);
+	print_x = SCALE_ENGINE_TO_GNOME_PRINT (x);
+	print_y = SCALE_ENGINE_TO_GNOME_PRINT (y);
 
-	gnome_print_gsave (printer->context);
+	cr = gtk_print_context_get_cairo_context (printer->context);
+
+	cairo_save (cr);
 
 	html_pango_get_item_properties (item, &properties);
-	
 	pango_glyph_string_extents (glyphs, item->analysis.font, NULL, &log_rect);
-	
-	if (properties.bg_color) {
-		gnome_print_setrgbcolor (printer->context,
+
+       	if (properties.bg_color) {
+		   cairo_set_source_rgb (cr,
 					 properties.bg_color->red / 65535.0,
 					 properties.bg_color->green / 65535.0,
 					 properties.bg_color->blue / 65535.0);
-		gnome_print_rect_filled (printer->context,
+		        cairo_rectangle (cr,
 					 print_x,
-					 print_y - SCALE_PANGO_TO_GNOME_PRINT (log_rect.y + log_rect.height),
-					 SCALE_PANGO_TO_GNOME_PRINT (log_rect.width),
-					 SCALE_PANGO_TO_GNOME_PRINT (log_rect.height));
+					 print_y + pango_units_to_double (log_rect.y + log_rect.height),
+					 pango_units_to_double (log_rect.width),
+					 pango_units_to_double (log_rect.height));
+			cairo_fill (cr);
 	}
 	
 	if (properties.fg_color) {
-		gnome_print_setrgbcolor (printer->context,
+		cairo_set_source_rgb     (cr,
 					 properties.fg_color->red / 65535.0,
 					 properties.fg_color->green / 65535.0,
 					 properties.fg_color->blue / 65535.0);
 	} else {
-		gnome_print_setrgbcolor (printer->context, 0., 0., 0.);
+		        cairo_set_source_rgb (cr, 0., 0., 0.);
 	}
-	
-	gnome_print_moveto (printer->context, print_x, print_y);
-	gnome_print_pango_glyph_string (printer->context, 
-					item->analysis.font,
-					glyphs);
+	cairo_move_to (cr, print_x, print_y);
+	pango_cairo_show_glyph_string (cr,
+				       item->analysis.font,
+				       glyphs);  
 	draw_lines (printer, print_x, print_y,
-		    SCALE_PANGO_TO_GNOME_PRINT (log_rect.width),
+		    pango_units_to_double (log_rect.width),
 		    &item->analysis, &properties);
 	
-	gnome_print_grestore (printer->context);
-	
-	
+	cairo_restore (cr);
 	return log_rect.width;
 }
 
@@ -655,21 +546,22 @@ draw_embedded (HTMLPainter *p, HTMLEmbedded *o, gint x, gint y)
 	gdouble print_x, print_y;	
 	HTMLPrinter *printer = HTML_PRINTER(p);
 	GtkWidget *embedded_widget;
+	cairo_t *cr;
 
-	html_printer_coordinates_to_gnome_print (printer, x, y, &print_x, &print_y);
-	gnome_print_gsave(printer->context); 
+	print_x = SCALE_ENGINE_TO_GNOME_PRINT (x);
+	print_y = SCALE_ENGINE_TO_GNOME_PRINT (y);
 
-	gnome_print_translate(printer->context, 
-			      print_x, print_y - o->height * PIXEL_SIZE);
- 
+	cr = gtk_print_context_get_cairo_context (printer->context);
+	cairo_save (cr);
+	cairo_translate (cr, 
+			print_x, print_y + o->height * PIXEL_SIZE);
 	embedded_widget = html_embedded_get_widget(o);
 	if (embedded_widget && GTK_IS_HTML_EMBEDDED (embedded_widget)) {
 		g_signal_emit_by_name(G_OBJECT (embedded_widget),
 				      "draw_print", 
-				      printer->context);
+				       cr);
 	}
-
-	gnome_print_grestore(printer->context); 
+	cairo_restore (cr);
 }
 
 static void
@@ -706,28 +598,23 @@ get_page_height (HTMLPainter *painter, HTMLEngine *e)
 }
 
 static void
-html_printer_init (GObject *object)
+html_printer_init (HTMLPrinter *printer)
 {
-	HTMLPrinter *printer;
-
-	printer                = HTML_PRINTER (object);
-	printer->context       = NULL;
-	printer->config        = NULL;
-
 	html_printer_set_scale (printer, 1.0);
 }
 
 static void
-html_printer_class_init (GObjectClass *object_class)
+html_printer_class_init (HTMLPrinterClass *class)
 {
+	GObjectClass *object_class;
 	HTMLPainterClass *painter_class;
 
-	painter_class = HTML_PAINTER_CLASS (object_class);
-
-	parent_class = g_type_class_ref (HTML_TYPE_PAINTER);
+	parent_class = g_type_class_peek_parent (class);
 
+	object_class = G_OBJECT_CLASS (class);
 	object_class->finalize = finalize;
 
+	painter_class = HTML_PAINTER_CLASS (class);
 	painter_class->begin = begin;
 	painter_class->end = end;
 	painter_class->alloc_color = alloc_color;
@@ -753,45 +640,46 @@ html_printer_class_init (GObjectClass *object_class)
 GType
 html_printer_get_type (void)
 {
-	static GType html_printer_type = 0;
+	static GType type = 0;
 
-	if (html_printer_type == 0) {
-		static const GTypeInfo html_printer_info = {
+	if (G_UNLIKELY (type == 0)) {
+		static const GTypeInfo type_info = {
 			sizeof (HTMLPrinterClass),
-			NULL,
-			NULL,
+			(GBaseInitFunc) NULL,
+			(GBaseFinalizeFunc) NULL,
 			(GClassInitFunc) html_printer_class_init,
-			NULL,
-			NULL,
+			(GClassFinalizeFunc) NULL,
+			NULL,  /* class_data */
 			sizeof (HTMLPrinter),
-			1,
+			0,     /* n_preallocs */
 			(GInstanceInitFunc) html_printer_init,
+			NULL   /* value_table */
 		};
-		html_printer_type = g_type_register_static (HTML_TYPE_PAINTER, "HTMLPrinter", &html_printer_info, 0);
+
+		type = g_type_register_static (
+			HTML_TYPE_PAINTER, "HTMLPrinter", &type_info, 0);
 	}
 
-	return html_printer_type;
+	return type;
 }
 
 HTMLPainter *
-html_printer_new (GtkWidget *widget, GnomePrintContext *context, GnomePrintJob *master)
+html_printer_new (GtkWidget *widget, GtkPrintContext *context)
 {
-	PangoFontMap *font_map;
-	HTMLPrinter *new;
+	const PangoFontDescription *desc;
+	HTMLPrinter *printer;
 	HTMLPainter *painter;
 
-	new = g_object_new (HTML_TYPE_PRINTER, NULL);
-	painter = HTML_PAINTER (new);
+	printer = g_object_new (HTML_TYPE_PRINTER, NULL);
+	printer->context = g_object_ref (context);
 
-	g_object_ref (context);
-	html_painter_set_widget (HTML_PAINTER (new), widget);
-	new->context = context;
-	new->master = master;
+	painter = HTML_PAINTER (printer);
+	html_painter_set_widget (painter, widget);
+	desc = pango_font_description_from_string ("sans 8");/* FIXME font hardcoded*/
+	painter->pango_context = gtk_print_context_create_pango_context (context);
+	pango_context_set_font_description (painter->pango_context, desc);
+	return painter; 
 
-	font_map = gnome_print_pango_get_default_font_map ();
-	painter->pango_context = gnome_print_pango_create_context (font_map);
-	
-	return HTML_PAINTER (new);
 }
 
 
@@ -804,7 +692,7 @@ html_printer_get_page_width (HTMLPrinter *printer)
 	g_return_val_if_fail (printer != NULL, 0);
 	g_return_val_if_fail (HTML_IS_PRINTER (printer), 0);
 
-	printer_width = printer_get_page_width (printer) - get_lmargin (printer) - get_rmargin (printer);
+	printer_width = printer_get_page_width (printer);
 	engine_width = SCALE_GNOME_PRINT_TO_ENGINE (printer_width);
 
 	return engine_width;
@@ -819,7 +707,7 @@ html_printer_get_page_height (HTMLPrinter *printer)
 	g_return_val_if_fail (printer != NULL, 0);
 	g_return_val_if_fail (HTML_IS_PRINTER (printer), 0);
 
-	printer_height = printer_get_page_height (printer) - get_tmargin (printer) - get_bmargin (printer);
+	printer_height = printer_get_page_height (printer);
 	engine_height = SCALE_GNOME_PRINT_TO_ENGINE (printer_height);
 
 	return engine_height;
@@ -844,5 +732,5 @@ html_printer_set_scale (HTMLPrinter *printer, gdouble scale)
 	painter = HTML_PAINTER (printer);
 	
 	printer->scale = scale;
-	painter->engine_to_pango = scale * PANGO_SCALE / 1024.;
+	painter->engine_to_pango = scale;
 }
